Package: pcaExplorer
Type: Package
Title: Interactive Visualization of RNA-seq Data Using a Principal
        Components Approach
Version: 3.6.0
Date: 2025-12-01
Authors@R: c(person("Federico", "Marini", role = c("aut", "cre"), 
    email ="marinif@uni-mainz.de", 
    comment = c(ORCID = '0000-0003-3252-7758')))
Description: This package provides functionality for interactive visualization 
    of RNA-seq datasets based on Principal Components Analysis. The methods provided
    allow for quick information extraction and effective data exploration. A Shiny
    application encapsulates the whole analysis.
License: MIT + file LICENSE
Imports: DESeq2, SummarizedExperiment, mosdef (>= 1.1.0),
        GenomicRanges, IRanges, S4Vectors, genefilter, ggplot2 (>=
        2.0.0), heatmaply, plotly, scales, NMF, plyr, topGO, limma,
        GOstats, GO.db, AnnotationDbi, shiny (>= 0.12.0),
        shinydashboard, shinyBS, ggrepel, DT, shinyAce, threejs,
        biomaRt, pheatmap, knitr, rmarkdown, base64enc, tidyr,
        grDevices, methods
Suggests: testthat, BiocStyle, markdown, airway, org.Hs.eg.db,
        htmltools
URL: https://github.com/federicomarini/pcaExplorer,
        https://federicomarini.github.io/pcaExplorer/
BugReports: https://github.com/federicomarini/pcaExplorer/issues
biocViews: ImmunoOncology, Visualization, RNASeq, DimensionReduction,
        PrincipalComponent, QualityControl, GUI, ReportWriting,
        ShinyApps
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/pcaExplorer
git_branch: RELEASE_3_23
git_last_commit: 27ac356
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-28 23:50:24 UTC; biocbuild
Author: Federico Marini [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3252-7758>)
Maintainer: Federico Marini <marinif@uni-mainz.de>
